@@ -13,6 +13,7 @@ import sys
 import socket
 import time
 import select
+import traceback
 
 from policyd_rate_limit import utils
 from policyd_rate_limit.utils import config
@@ -26,6 +27,7 @@ class Policyd(object):
     """The policy server class"""
     socket_data_read = {}
     socket_data_write = {}
+    last_used = {}
 
     def socket(self):
         """initialize the socket from the config parameters"""
@@ -71,6 +73,19 @@ class Policyd(object):
             pass
         connection.close()
 
+    def close_write_conn(self, connection):
+        """Removes a socket from the write dict"""
+        try:
+            del self.socket_data_write[connection]
+        except KeyError:
+            if config.debug:
+                sys.stderr.write(
+                    (
+                        "Hmmm, a socket actually used to write a little "
+                        "time ago wasn\'t in socket_data_write. Weird.\n"
+                    )
+                )
+
     def run(self):
         """The main server loop"""
         try:
@@ -96,6 +111,9 @@ class Policyd(object):
                             sys.stderr.write('connection from %s\n' % (client_address,))
                             sys.stderr.flush()
                         self.socket_data_read[connection] = []
+
+                        # Updates the last_sed time for the socket.
+                        self.last_used[connection] = time.time()
                     # else there is data to read on a client socket
                     else:
                         self.read(socket)
@@ -107,10 +125,24 @@ class Policyd(object):
                         if data_not_sent:
                             self.socket_data_write[socket] = data_not_sent
                         else:
-                            self.close_connection(socket)
+                            self.close_write_conn(socket)
+
+                        # Socket has been used, let's update its last_used time.
+                        self.last_used[socket] = time.time()
                     # the socket has been closed during read
                     except KeyError:
                         pass
+                # Closes unused socket for a long time.
+                __to_rm = []
+                for (socket, last_used) in self.last_used.items():
+                    if socket == sock:
+                        continue
+                    if time.time() - last_used > config.delay_to_close:
+                        self.close_connection(socket)
+                        __to_rm.append(socket)
+                for socket in __to_rm:
+                    self.last_used.pop(socket)
+
         except (KeyboardInterrupt, utils.Exit):
             for socket in list(self.socket_data_read.keys()):
                 if socket != self.sock:
@@ -154,11 +186,13 @@ class Policyd(object):
                 self.action(connection, request)
             else:
                 self.socket_data_read[connection] = buffer
+            # Socket has been used, let's update its last_used time.
+            self.last_used[connection] = time.time()
         except (KeyboardInterrupt, utils.Exit):
             self.close_connection(connection)
             raise
         except Exception as error:
-            sys.stderr.write("%s\n" % error)
+            traceback.print_exc()
             sys.stderr.flush()
             self.close_connection(connection)
 
@@ -180,6 +214,9 @@ class Policyd(object):
                     # if user is authenticated, we filter by sasl username
                     if config.limit_by_sasl and u'sasl_username' in request:
                         id = request[u'sasl_username']
+                    # else, if activated, we filter by sender
+                    elif config.limit_by_sender and u'sender' in request:
+                        id = request[u'sender']
                     # else, if activated, we filter by ip source addresse
                     elif (
                         config.limit_by_ip and
@@ -191,7 +228,7 @@ class Policyd(object):
                     # to the next section
                     else:
                         raise Pass()
-                    # Here we are limiting agains sasl username or ip source addresses.
+                    # Here we are limiting against sasl username, sender or source ip addresses.
                     # for each limit periods, we count the number of mails already send.
                     # if the a limit is reach, we change action to fail (deny the mail).
                     for mail_nb, delta in config.limits_by_id.get(id, config.limits):
@@ -232,3 +269,5 @@ class Policyd(object):
             sys.stderr.flush()
         # return the result to the client
         self.socket_data_write[connection] = data.encode('UTF-8')
+        # Socket has been used, let's update its last_used time.
+        self.last_used[connection] = time.time()
