@@ -268,7 +268,7 @@ class _cursor(object):
     def del_db(cls):
         try:
             cls._db[threading.current_thread()].close()
-        except:
+        except Exception as e:
             pass
         try:
             del cls._db[threading.current_thread()]
@@ -332,9 +332,12 @@ def clean():
         print("%d records deleted" % cur.rowcount)
         # if report is True, generate a mail report
         if config.report and config.report_to:
-            send_report(cur)
-            # The mail report has been successfully send, flush limit_report
-            cur.execute("DELETE FROM limit_report")
+            report_text = gen_report(cur)
+        # The mail report has been successfully send, flush limit_report
+        cur.execute("DELETE FROM limit_report")
+    # send report
+    if len(report_text) != 0:
+        send_report(report_text)
 
     try:
         if config.backend == PGSQL_DB:
@@ -355,10 +358,11 @@ def clean():
             cursor.get_db().autocommit = False
 
 
-def send_report(cur):
+def gen_report(cur):
     cur.execute("SELECT id, delta, hit FROM limit_report")
     # list to sort ids by hits
     report = list(cur.fetchall())
+    text = []
     if not config.report_only_if_needed or report:
         if report:
             text = ["Below is the table of users who hit a limit since the last cleanup:", ""]
@@ -368,7 +372,7 @@ def send_report(cur):
             for (id, delta, hit) in report:
                 report_d[id].append((delta, hit))
                 max_d['id'] = max(max_d['id'], len(id))
-                max_d['delta'] = max(max_d['delta'], len(str(delta)))
+                max_d['delta'] = max(max_d['delta'], len(str(delta)) + 1)
                 max_d['hit'] = max(max_d['hit'], len(str(hit)))
             # sort by hits
             report.sort(key=lambda x: x[2])
@@ -404,13 +408,10 @@ def send_report(cur):
         else:
             text = ["No user hit a limit since the last cleanup"]
         text.extend(["", "-- ", "policyd-rate-limit"])
+    return text
 
-        # Start building the mail report
-        msg = MIMEMultipart()
-        msg['Subject'] = config.report_subject or ""
-        msg['From'] = config.report_from or ""
-        msg['To'] = config.report_to
-        msg.attach(MIMEText("\n".join(text), 'plain'))
+
+def send_report(text):
 
         # check that smtp_server is wekk formated
         if isinstance(config.smtp_server, (list, tuple)):
@@ -436,8 +437,21 @@ def send_report(cur):
                     server.login(config.smtp_credentials[0], config.smtp_credentials[1])
                 else:
                     ValueError("bad smtp_credentials should be a tuple (login, password)")
-            server.sendmail(config.report_from or "", config.report_to, msg.as_string())
+
+            if not isinstance(config.report_to, list):
+                report_to = [config.report_to]
+            else:
+                report_to = config.report_to
+            for rcpt in report_to:
+                # Start building the mail report
+                msg = MIMEMultipart()
+                msg['Subject'] = config.report_subject or ""
+                msg['From'] = config.report_from or ""
+                msg['To'] = rcpt
+                msg.attach(MIMEText("\n".join(text), 'plain'))
+                server.sendmail(config.report_from or "", rcpt, msg.as_string())
         finally:
+            print('report is sent')
             server.quit()
 
 
@@ -538,4 +552,5 @@ def exit_signal_handler(signal, frame):
     Used for coverage computation"""
     raise Exit()
 
+
 config = LazyConfig()
